/* main.c - This file is a part of NutsOS
 *
 * NutsOS
 * Copyright (C) 2013  Free Software Foundation, Inc.
 *
 * NutsOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NutsOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with NutsOS; If not, see <http://www.gnu.org/licenses/>.
 *
 * Valentin Verdier <valentin.verdier03@gmail.com>
 */

#include "args.h"
#include "types.h"
#include "print.h"
#include "interrupt.h"
#include "utils.h"
#include "memory.h"
#include "asm_x86.h"
#include "process.h"


void main()
{
	arg = (struct ArgModel*) 0x1400;
	CurrentProcess = 0;

	print("\nNutsOS Kernel ( NOS-kernel32.bin build 2013-12-01 )\n", 0x6F);

	print("[Kernel] useable memory -> %d Ko\n", DEFAULT_COLOR, (arg->phyMemSize / 1024));

	if(arg->phyMemSize / 1024 < 10240)
	{
		print("ERROR : A USEABLE SYSTEM MUST HAVE MORE THAN 10 Mo", 0x47);
		asm("hlt");
	}

	print("[Kernel] initializing memory managment...", DEFAULT_COLOR);
	InitGDT();
	InitKernelSpace();
	_EnablePaging();
	print("ok\n", 0x02);

	print("[Kernel] setting up interrupts...", DEFAULT_COLOR);
	InitIDT();
	//setTimerDivid(11932); /* (1193181Hz / div) div = 1193 pour 1KHz, div = 11932 pour 100Hz */
	print("ok\n", 0x02);

	print("[Kernel] setting up processes scheduler...", DEFAULT_COLOR);
	InitProcessManager();
	print("ok\n", 0x02);

	print("[Kernel] end of initialisation, switch to the god process\n", DEFAULT_COLOR);
	CurrentProcess = &Process_0;
	tss->ss0 = 0x18;
	tss->esp0 = CurrentProcess->kernelStackPtr;
	_LoadContext(CurrentProcess);

	/*print("[Kernel] detect PCI devices...\n", DEFAULT_COLOR);
	PciDetectDevices();
	print("done\n", DEFAULT_COLOR);*/

	/* Normalement, c'est impossible d'arriver jusque ici */
	while(1)
	{
		asm("hlt");
	}
}
