/* scheduler.c - This file is a part of NutsOS
 *
 * NutsOS
 * Copyright (C) 2013  Free Software Foundation, Inc.
 *
 * NutsOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NutsOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with NutsOS; If not, see <http://www.gnu.org/licenses/>.
 *
 * Valentin Verdier <valentin.verdier03@gmail.com>
 */

#include "process.h"

#include "types.h"
#include "asm_x86.h"
#include "interrupt.h"
#include "signal.h"

T_Process* SchedulingAlgorithm()
{
	T_Process *process = CurrentProcess;

	do
	{
		if(process->next)
		{
			process = process->next;
		}
		else
		{
			process = (T_Process*) Processes.firstEntry;
		}

		if(process != CurrentProcess)
		{
			unsigned int i;
			for(i = 0; i < 32; i++)
			{
				if(FetchSignal(i, process))
				{
					HandleSignal(i, process);
					break;
				}
			}

			if(process->status == PROCESS_STATUS_RUNNING)
			{
				return process;
			}
			else if(process->status == PROCESS_STATUS_WAIT_MUTEX) /* Si on attend un mutex */
			{
				/* On regard son etat  */
				if(!(*process->ipcWaitMutex))
				{
					process->status = PROCESS_STATUS_RUNNING;
					*process->ipcWaitMutex = 1;
					process->ipcWaitMutex = 0;
					process->ipcInSection++;
					return process;
				}
			}
			/*else if(process->status == PROCESS_STATUS_SLEEP) si le processus est en sommeil
			{
				On regard si un wakeup à été envoyé
			}*/
		}
		else
		{
			break;
		}
	}while(1);

	if(Process_0.status == PROCESS_STATUS_RUNNING)
	{
		return &Process_0;
	}
	else
	{
		return CurrentProcess;
	}
}

void Schedule()
{
	bits32 *context = _SaveContex();
	bits32 *curStackFrame = (bits32*) _GetStackFrameAddr(0);
	_ClearInterrupt();

	CurrentProcess->reg_GS = (bits16) context[0];
	CurrentProcess->reg_FS = (bits16) context[1];
	CurrentProcess->reg_ES = (bits16) context[2];
	CurrentProcess->reg_DS = (bits16) context[3];
	CurrentProcess->reg_EDI = context[4];
	CurrentProcess->reg_ESI = context[5];
	CurrentProcess->reg_EBP = curStackFrame[0]; /* ebp de la fonction précédente */
	CurrentProcess->reg_EBX = context[8];
	CurrentProcess->reg_EDX = context[9];
	CurrentProcess->reg_ECX = context[10];
	CurrentProcess->reg_EAX = context[11];
	CurrentProcess->reg_EIP = curStackFrame[1]; /* eip de retour */
	CurrentProcess->reg_CS = 0x08;
	CurrentProcess->reg_EFLAGS = (context[12] | 0x200) & 0xFFFFBFFF;
	CurrentProcess->reg_ESP = (bits32) &curStackFrame[2];
	CurrentProcess->reg_SS = 0x18;

	CurrentProcess = SchedulingAlgorithm();

	tss->ss0 = 0x18;
	tss->esp0 = CurrentProcess->kernelStackPtr;
	_LoadContext(CurrentProcess);
}

void ScheduleAtClock()
{
	if(!CurrentProcess && Processes.entryCount > 0)
	{
		CurrentProcess = (T_Process*) Processes.firstEntry;
	}
	else if(Processes.entryCount <= 1)
	{
		return;
	}
	else if(Processes.entryCount > 1)
	{
		bits32 *context = (bits32*) (_GetStackFrameAddr(1) + 8);

		/* Registre empilés manuellement */
		CurrentProcess->reg_GS = (bits16) context[0];
		CurrentProcess->reg_FS = (bits16) context[1];
		CurrentProcess->reg_ES = (bits16) context[2];
		CurrentProcess->reg_DS = (bits16) context[3];
		CurrentProcess->reg_EDI = context[4];
		CurrentProcess->reg_ESI = context[5];
		CurrentProcess->reg_EBP = context[6];
		/* ESP context[7] inutile */
		CurrentProcess->reg_EBX = context[8];
		CurrentProcess->reg_EDX = context[9];
		CurrentProcess->reg_ECX = context[10];
		CurrentProcess->reg_EAX = context[11];

		/* registres empliés automatiquement */
		CurrentProcess->reg_EIP = context[12];
		CurrentProcess->reg_CS = (bits16) context[13];
		CurrentProcess->reg_EFLAGS = (context[14] | 0x200) & 0xFFFFBFFF;

		if(CurrentProcess->reg_CS == 0x08)
		{
			CurrentProcess->reg_ESP = (bits32) &context[15];
			CurrentProcess->reg_SS = 0x18;
		}
		else
		{
			CurrentProcess->reg_ESP = context[15];
			CurrentProcess->reg_SS = (bits16) context[16];
		}

		CurrentProcess = SchedulingAlgorithm();
	}
	tss->ss0 = 0x18;
	tss->esp0 = CurrentProcess->kernelStackPtr;
	PIC_Send_EOI();
	_LoadContext(CurrentProcess);
}
