/* signal.c - This file is a part of NutsOS
 *
 * NutsOS
 * Copyright (C) 2013  Free Software Foundation, Inc.
 *
 * NutsOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * NutsOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with NutsOS; If not, see <http://www.gnu.org/licenses/>.
 *
 * Valentin Verdier <valentin.verdier03@gmail.com>
 */

#include "signal.h"

#include "process.h"

void SendSignal(unsigned int num, T_Process* process)
{
	if(num < 32)
	{
		process->signal |= (1 << num);
	}
}

void ClearSignal(unsigned int num, T_Process* process)
{
	if(num < 32)
	{
		process->signal &= ~(1 << num);
	}
}

unsigned int FetchSignal(unsigned int num, T_Process* process)
{
	if(num < 32)
	{
		if(process->signal & (1 << num))
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
}

void HandleSignal(unsigned int signal, T_Process* process)
{
	switch(signal)
	{
		case SIGNAL_KILL:
			print("Process with PID %d received the signal 0 (KILL)\n", DEFAULT_COLOR, process->pid);
			break;

		case SIGNAL_TERM:
			print("Process with PID %d received the signal 1 (TERM)\n", DEFAULT_COLOR, process->pid);
			break;

		case SIGNAL_KCHILD:
			print("Process with PID %d received the signal 2 (KCHILD)\n", DEFAULT_COLOR, process->pid);
			break;

		default:
			print("Process with PID %d received an unknown signal\n", DEFAULT_COLOR, process->pid);
			break;
	}

	ClearSignal(signal, process);
}
